#!/bin/bash

## This is the script jenkins should run to provision a device in the lab

set -e

BASEDIR=$(dirname $(readlink -f $0))

RESDIR=`pwd`/clientlogs
ZIPDIR=`pwd`/archive

NETWORK_FILE="${NETWORK_FILE-/home/ubuntu/magners-wifi}"

IMAGE_OPT="${IMAGE_OPT-ubuntu-system -b --channel trusty-proposed}"
UUID="${UUID-$(uuidgen -r)}"

usage() {
cat <<EOF
usage: $0 [-s ANDROID_SERIAL] [-n NETWORK_FILE] [-P ppa] [-p package] [-w]

Provisions the given device with the latest build

OPTIONS:
  -h	Show this message
  -s    Specify the serial of the device to install
  -n    Select network file
  -P    add the ppa to the target (can be repeated)
  -p    add the package to the target (can be repeated)
  -a    add a URL for a jenkins created zip archive of packages
  -w    make the system writeable (implied with -p and -P arguments)
  -c    Specify the click package to install
  -C    Specify the source directory for click tests

EOF
}

image_info() {
	# mark the version we installed in /home/phablet/.ci-[uuid,flash-args]
	# adb shell messes up \n's with \r\n's so do the whole of the regex on the target
	IMAGEVER=$(adb shell "system-image-cli -i | sed -n -e 's/version version: \([0-9]*\)/\1/p' -e 's/version ubuntu: \([0-9]*\)/\1/p' -e 's/version device: \([0-9]*\)/\1/p' | paste -s -d:")
	CHAN=$(adb shell "system-image-cli -i | sed -n -e 's/channel: \(.*\)/\1/p' | paste -s -d:")
	REV=$(echo $IMAGEVER | cut -d: -f1)
	echo "$IMAGE_OPT" | grep -q "\-\-revision" || IMAGE_OPT="${IMAGE_OPT} --revision $REV"
	echo "$IMAGE_OPT" | grep -q "\-\-channel" || IMAGE_OPT="${IMAGE_OPT} --channel $CHAN"
	adb shell "echo '${IMAGEVER}' > /home/phablet/.ci-version"
	echo $UUID > $RESDIR/.ci-uuid
	adb push $RESDIR/.ci-uuid /home/phablet/
	cat > $RESDIR/.ci-flash-args <<EOF
$IMAGE_OPT
EOF
	adb push $RESDIR/.ci-flash-args /home/phablet/.ci-flash-args
	echo $CUSTOMIZE > $RESDIR/.ci-customizations
	adb push $RESDIR/.ci-customizations /home/phablet/.ci-customizations
}

while getopts i:s:n:P:p:a:c:C:wh opt; do
	case $opt in
	h)
		usage
		exit 0
		;;
	n)
		NETWORK_FILE=$OPTARG
		;;
	s)
		export ANDROID_SERIAL=$OPTARG
		;;
	i)
		IMAGE_TYPE=$OPTARG
		;;
	w)
		# making this a non-zero length string enables the logic
		CUSTOMIZE=" "
		;;
	P)
		CUSTOMIZE="$CUSTOMIZE --ppa $OPTARG"
		;;
	p)
		CUSTOMIZE="$CUSTOMIZE -p $OPTARG"
		;;
	a)
		ARCHIVE="$OPTARG"
		;;
        c)
		CLICK_PACKAGE="$OPTARG"
		;;
        C)
		CLICK_SOURCE="$OPTARG"
		;;
	esac
done

if [ -z $ANDROID_SERIAL ] ; then
	# ensure we only have one device attached
	lines=$(adb devices | wc -l)
	if [ $lines -gt 3 ] ; then
		echo "ERROR: More than one device attached, please use -s option"
		echo
		usage
		exit 1
	fi
fi

set -x
[ -d $RESDIR ] && rm -rf $RESDIR
mkdir -p $RESDIR
[ -d $ZIPDIR ] && rm -rf $ZIPDIR

# Setup the local archive bits first, exit out before flashing if it fails
if [ -n "$ARCHIVE" ]; then
    wget $ARCHIVE
    ZIPNAME=`basename $ARCHIVE`
    unzip $ZIPNAME -d $ZIPDIR
    rm $ZIPNAME
    if [ -d $ZIPDIR/archive/work/output ]; then
        ARCHIVEDIR=$ZIPDIR/archive/work/output
    elif [ -e $ZIPDIR/output ]; then
        ARCHIVEDIR=$ZIPDIR/output
        chmod 755 $ARCHIVEDIR
    else
        echo "E: Cannot find local package archive" >&2
        exit 1
    fi
    cd $ARCHIVEDIR
    dpkg-scanpackages . /dev/null | gzip -9c > Packages.gz
    cd -
fi

if [ -n "$SKIP_FLASH" ]; then
    echo "SKIPPING phablet-flash AS REQUESTED"
else
    phablet-flash $IMAGE_OPT
    adb wait-for-device
    sleep 20  #give the system a little time
fi

CLICK_PACKAGE_NAME=""
CLICK_PACKAGE_SETUP=""
if [ -n "$CLICK_PACKAGE" ]; then
    # Copy click package to device and install
    CLICK_SERIAL=""
    if [ -n "$ANDROID_SERIAL" ]; then
        CLICK_SERIAL="--serial $ANDROID_SERIAL"
    fi
    CLICK_BASE_NAME=`basename ${CLICK_PACKAGE}`
    adb push "${CLICK_PACKAGE}" /home/phablet/
    adb shell "chown phablet:phablet /home/phablet/${CLICK_BASE_NAME}"

    CLICK_PACKAGE_NAME=`$BASEDIR/click-info.py $CLICK_SERIAL --key name /home/phablet/${CLICK_BASE_NAME}`
    CLICK_PACKAGE_VERSION=`$BASEDIR/click-info.py $CLICK_SERIAL --key version /home/phablet/${CLICK_BASE_NAME}`
    CLICK_PACKAGE_SETUP="--click ${CLICK_BASE_NAME}"
    adb shell "click install --user phablet /home/phablet/${CLICK_BASE_NAME}"
    adb shell "click register --user phablet ${CLICK_PACKAGE_NAME} ${CLICK_PACKAGE_VERSION}"
fi

if [ -n "$SKIP_CLICK" ]; then
    echo "SKIPPING phablet-click-test-setup AS REQUESTED"
else
    phablet-click-test-setup $CLICK_PACKAGE_SETUP
fi

if [ -n "$CLICK_SOURCE" ]; then
    # Copy click tests to device
    # phablet-click-test-setup install tests under /home/phablet/autopilot
    AUTOPILOTFILES=`ls ${CLICK_SOURCE}/tests/autopilot/`
    for DIR in $AUTOPILOTFILES; do
        if [ -d "${CLICK_SOURCE}/tests/autopilot/$DIR" ]; then
            adb shell "rm -rf /home/phablet/autopilot/${DIR}"
            adb push ${CLICK_SOURCE}/tests/autopilot/${DIR} /home/phablet/autopilot/${DIR}
            adb shell "chown -R phablet:phablet /home/phablet/autopilot"
        fi
    done
fi

phablet-network -n $NETWORK_FILE

if [ "$IMAGE_TYPE" = "touch_sf4p" ]; then
    adb shell rm -f /home/phablet/.display-mir
fi

phablet-config edges-intro --disable

# get our target-based utilities into our PATH
adb push ${BASEDIR}/../utils/target /home/phablet/bin

image_info

# Setup the provided archive on the device
if [ -n "$ARCHIVE" ]; then
    phablet-config writable-image
    adb push $ARCHIVEDIR /home/phablet/archive
    adb push ${BASEDIR}/../support/00localrepo.list /etc/apt/sources.list.d
    adb push ${BASEDIR}/../support/local-pin-1100 /etc/apt/preferences.d
fi

if [ -n "$CUSTOMIZE" ] ; then
	echo "= CUSTOMIZING IMAGE"
	phablet-config writable-image $CUSTOMIZE
        # Make sure whoopsie-upload-all can work (bug #1245524)
        adb shell "sed -i '/Waiting for whoopsie/ a\ subprocess.call([\"restart\", \"whoopsie\"])' /usr/share/apport/whoopsie-upload-all"
fi
